#!/bin/bash
#
#  upnp2mrtg - Monitoring AVM Fritz!Box With MRTG
#  This versions have been reported as working: 3030, 5050, 7050, 7141 and 7170
#
#  Copyright (C) 2005 Michael Tomschitz <michael.tomschitz@ANetzB.de>
#  modularized by Mirko Dölle <mid@ct.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#  The latest version of this software can be obtained here:
#
#  http://www.ANetzB.de/upnp2mrtg/
#
#  $Id: upnp2mrtg,v 1.9l 2019/04/15 12:00:00 mcedit Exp $

# functions
request_header () {
  echo "POST ${4} HTTP/1.0
HOST: ${1}:${2}
CONTENT-LENGTH: ${3}
CONTENT-TYPE: text/xml; charset=\"utf-8\"
SOAPACTION: \"${5}#${6}\""
}

soap_form () {
  echo "<?xml version=\"1.0\" encoding=\"utf-8\"?>
<s:Envelope
        xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\"
        s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">
    <s:Body>
        <u:${1} xmlns:u=\"urn:schemas-upnp-org:service:${2}:1\" />
    </s:Body>
</s:Envelope>"
}

soapreq () {
  stype=$1
  url=$2
  shift 2
  request="`soap_form ${@}`"
  post="`request_header $HOST $PORT ${#request} ${url} ${stype} ${@}`

$request"
  rs="`get_response "$post"`"
  if [ $? -eq 0 ]; then
    echo "`get_attribute "$2" "$rs"`"
  fi
}

get_attribute () {
    _get_attribute_start_tag () { echo "${2#*<$1>*}"; }
    _get_attribute_end_tag () { echo "${2%*</$1>*}"; }
    _get_attribute_tag () { _get_attribute_start_tag "$1" "`_get_attribute_end_tag $1 "$2"`" ; }
    _get_attribute_num () { echo $#; }
    if [ `_get_attribute_num $1` -gt 1 ]; then
        get_attribute "${1#* }" "`_get_attribute_tag "${1%% *}" "$2"`"
    else
        _get_attribute_tag "${1}" "$2"
    fi
}

modulo_time () {
  echo "$((${1} / ${2})) $((${1} % ${2}))"
}

shell_netcat () {
  exec 5<>/dev/tcp/$1/$2; cat >&5; cat <&5;
}

get_response () {
  _get_response_rs="`echo "$1" | $nc $HOST $PORT 2>/dev/null`"
  _get_response_rv=$?
  echo "$_get_response_rs"
  if ${VERBOSE:-false}; then
    echo
    echo "---- REQUEST: ----">&2
    echo "$1">&2
    echo "---- RESPONSE: ----">&2
    echo "$_get_response_rs">&2
    echo "----">&2
  fi
  return $_get_response_rv
}

request_header_http () {
  echo "GET $3 HTTP/1.0

"
}
